/* Example C Player for MIT Programming Challenge 1998 */

/***** Includes *****/

#include <string.h>		/* for strcmp(), memset(), etc. */
#include <stdlib.h>		/* for atoi(), rand(), etc.     */
#include <stdio.h>		/* for sprintf()                */
#include <time.h>		/* for time()                   */

#include "player_interface.h"	/* for Mitosis definitions      */


/***** Global variables *****/

/* Initialize this variable to a name or nickname */
/* you would like to be displayed for your player. */
char      Player_Name[17] = "Example player";


/***** Procedure declarations *****/

/* Player routine */
void      Next_Play_Request(Entity_State * life_form_state_ptr,
			    Next_Play * next_play_ptr);

/* Diagnostic routines */
void      Next_Play_Print(Next_Play * play_ptr, int round_num, int life_id);
void      Look_Info_Print(Look_Info * player_look_info_ptr);
void      State_Print(Entity_State * life_form_state_ptr);


/***** Procedures *****/

/** When your entity's turn comes up, this function will    **/
/** be called to request its next play choice.              **/
/** Replace the body of this function with your own logic.  **/
void      Next_Play_Request(Entity_State * life_form_state_ptr,
			    Next_Play * next_play_ptr)
{
    static int random_seeded = 0;

    if (!random_seeded) {
	srandom(getpid());
	random_seeded = 1;
    }
    /* Split every 50 rounds, if the entity's energy is 100 or more. */
    if (life_form_state_ptr->round_number % 100 == 50
	    && life_form_state_ptr->energy >= 100) {
	next_play_ptr->play_choice = Split;
    }
    /* Look approximately every 10th round; display a message when doing so. */
    else if (life_form_state_ptr->round_number % 10 == random() % 10) {
	next_play_ptr->play_choice = Look;
	next_play_ptr->look_radius = 2;

	next_play_ptr->message_flag = 1;
	sprintf(next_play_ptr->message1, "Looking...");
    }
    /* Otherwise, move in a random direction. */
    else {
	next_play_ptr->play_choice = Move_Up + (random() % 4);
    }
}


/** The following procedures are provided for diagnostic use.   **/
/** They print to standard output a text representation of the  **/
/** data structures used by the interface to the Mitosis game.  **/

/** Print out the contents of a "Next play" data structure **/
void      Next_Play_Print(Next_Play * play_ptr, int round_num, int life_id)
{
    static char *play_text[] =
    {
	"Stay",
	"Move_Up",
	"Move_Down",
	"Move_Left",
	"Move_Right",
	"Split",
	"Look"
    };
    char      look_distance_text[16];

    if (play_ptr->play_choice == Look) {
	sprintf(look_distance_text, "%d", play_ptr->look_radius);
    } else {
	sprintf(look_distance_text, "");
    }

    printf("Round #%d: Player %s: life ID # %d: %s %s\n",
	   round_num, Player_Name, life_id,
	   play_text[play_ptr->play_choice], look_distance_text);
    if (play_ptr->message_flag) {
	printf("    Message print request -\n");
	printf("        %s\n", play_ptr->message1);
	printf("        %s\n", play_ptr->message2);
    }
}


/** Print out the contents of an "Entity State" data structure **/
void      State_Print(Entity_State * life_form_state_ptr)
{
    printf("Entity State data structure:\n");
    printf("============================\n");
    printf("Round                %d\n", life_form_state_ptr->round_number);
    printf("Entity ID            # %d (at %d, %d)\n",
	   life_form_state_ptr->entity_ID,
	   life_form_state_ptr->entity_position_x,
	   life_form_state_ptr->entity_position_y);
    if (life_form_state_ptr->not_dead_yet_flag == 0) {
	printf("This entity is now dead.\n");
    }
    printf("  energy             %d\n", life_form_state_ptr->energy);
    printf("Sensor\n");
    printf("  food               %f\n", life_form_state_ptr->sense_of_food);
    printf("  toxin              %f\n", life_form_state_ptr->sense_of_toxin);
    printf("  foes               %f\n", life_form_state_ptr->sense_of_foes);
    if (life_form_state_ptr->square_contents == Food) {
	printf("Current square       Food\n");
    } else if (life_form_state_ptr->square_contents == Toxin) {
	printf("Current square       Toxin\n");
    } else if (life_form_state_ptr->square_contents == Empty) {
	printf("Current square       Empty\n");
    }
    if (life_form_state_ptr->look_info_ptr != NULL) {
	printf("Entity is looking\n");
    }
}

/** Print out the contents of a "Look Info" data structure **/
void      Look_Info_Print(Look_Info * player_look_info_ptr)
{
    int       x, y;
    int       side_length;

    side_length = player_look_info_ptr->side_length;

    printf("Look Information data structure:\n");
    printf("================================\n");
    printf("Area of look information covers\n");
    printf("the board from (%d, %d) to (%d, %d).\n",
	   player_look_info_ptr->board_x, player_look_info_ptr->board_y,
	   player_look_info_ptr->board_x + side_length - 1,
	   player_look_info_ptr->board_y + side_length - 1);

    printf("Key:\n");
    printf(".. = Empty\n");
    printf("## = Wall\n");
    printf("** = Friend\n");
    printf("XX = Foe\n");
    printf("oo = Food\n");
    printf("TT = Toxin\n");
    printf("TX = Toxin_And_Foe\n");
    printf("T* = Toxin_And_Friend\n");

    for (y = 0; y < side_length; y++) {
	for (x = 0; x < side_length; x++) {
	    switch (player_look_info_ptr->squares[y][x]) {
	    case Empty:
		{
		    printf(".. ");
		    break;
		}
	    case Wall:
		{
		    printf("## ");
		    break;
		}
	    case Friend:
		{
		    printf("** ");
		    break;
		}
	    case Foe:
		{
		    printf("XX ");
		    break;
		}
	    case Food:
		{
		    printf("oo ");
		    break;
		}
	    case Toxin:
		{
		    printf("TT ");
		    break;
		}
	    case Toxin_And_Foe:
		{
		    printf("TX ");
		    break;
		}
	    case Toxin_And_Friend:
		{
		    printf("T* ");
		    break;
		}
	    }
	}

	printf("\n");
    }

    printf("\n");
}
